from django.conf import settings
from django.conf.urls.static import static
from django.views.generic.base import RedirectView
from django.conf.urls import url
from src.tipboard.app.properties import *
from src.tipboard.app.views.api import push, projectInfo,  tile, meta, update#, deleteTile, getTile
from src.tipboard.app.views.dashboard import dashboardRendererHandler, getDashboardsPaths, flipboardHandler


favicon_view = RedirectView.as_view(url="/static/" + 'favicon.ico', permanent=True)

urlpatterns = [
    url(r'^flipboard/getDashboardsPaths$', getDashboardsPaths),
    url(r'^$', flipboardHandler),
    url(r'^([a-zA-Z0-9_-]*)$', dashboardRendererHandler),
    url(r'^api/' + API_VERSION + '/' + API_KEY + '/tiledata/([a-zA-Z0-9_-]+)$', tile),
    url(r'^api/' + API_VERSION + '/' + API_KEY + '/tileconfig/([a-zA-Z0-9_-]+)$', meta),
    url(r'^api/' + API_VERSION + '/' + API_KEY + '/info$', projectInfo),
    url(r'^api/' + API_VERSION + '/' + API_KEY + '/push$', push),
    url(r'^api/' + API_VERSION + '/' + API_KEY + '/update$', update),
] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)




import os
from src.tipboard.app.properties import SUPER_SECRET_KEY, debug, REDIS_HOST, REDIS_PORT, FROM_PIP

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
SECRET_KEY = SUPER_SECRET_KEY
DEBUG = debug
ALLOWED_HOSTS = ['*']

print("LOADING:-> " +  FROM_PIP + 'tipboard.app.Config')

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'channels',
    FROM_PIP + 'tipboard.app.Config'
]

MIDDLEWARE = [
    'django_prometheus.middleware.PrometheusBeforeMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.BrokenLinkEmailsMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'corsheaders.middleware.CorsPostCsrfMiddleware',
    'django_prometheus.middleware.PrometheusAfterMiddleware',
]

CORS_ORIGIN_ALLOW_ALL = True
CORS_ALLOW_CREDENTIALS = True
CORS_ORIGIN_REGEX_WHITELIST = (
    '*',
)

CORS_ALLOW_METHODS = (
    'GET',
    'POST'
)

CORS_ALLOW_HEADERS = (
    'accept',
    'accept-encoding',
    'authorization',
    'multipart/form-data',
    'content-type',
    'dnt',
    'origin',
    'user-agent',
    'x-csrftoken',
    'x-requested-with',
    'Set-Cookie',
    'enctype',
)

LANGUAGE_CODE = 'fr-FR'
TIME_ZONE = 'Europe/Paris'
USE_I18N = True
USE_L10N = True
USE_TZ = True

ROOT_URLCONF = FROM_PIP + 'tipboard.webserver.urls'
WSGI_APPLICATION = FROM_PIP + 'tipboard.webserver.wsgi.application'

print("->>>" + BASE_DIR + '/templates')
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR + '/templates'],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        'libraries':{
            'template_filter': FROM_PIP + 'tipboard.templates.template_filter',
            }
        },
    },
]


STATICFILES_FINDERS = (
        'django.contrib.staticfiles.finders.FileSystemFinder',
        'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)
STATICFILES_DIRS = [
    os.path.join(BASE_DIR, 'static'),
]

STATIC_URL = '/static/'

ASGI_APPLICATION = FROM_PIP + 'tipboard.webserver.routing.application'

CHANNEL_LAYERS = {
    'default': {
        'BACKEND': 'channels_redis.core.RedisChannelLayer',
        'CONFIG': {
            'hosts': [(REDIS_HOST, REDIS_PORT)],
        },
    }
}




# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import time, os, json
from datetime import datetime
from src.tipboard.app.properties import PROJECT_NAME, TIPBOARD_PATH, LOG
from src.tipboard.app.parser import get_tiles_configs


def validate_post_request(post_field, allowed_fields):
    """
    Validators returning None if everything is OK - otherwise suitable HTTP
    status and message is set.
    """
    error_msg = None
    dict_diff = list(set(post_field) - set(allowed_fields))
    if dict_diff:
        error_msg = 'There are fields not allowed in your request.'
    for field in allowed_fields:
        if field not in post_field.keys():
            error_msg = "There is no '{}' field in your request.".format(
                field,
            )
            break
    return error_msg


def validate_with_config_file(post, parsed_config):
    error_msg = None
    if post['tile'][0] not in parsed_config['tiles_names']:
        error_msg = "Tile's name not found in the configuration file.\n"
    elif post['key'][0] not in parsed_config['tiles_keys']:
        error_msg = "Tile's key/id not found in the configuration file.\n"
    return error_msg


def validate_with_config_files(post):
    tiles_configs = get_tiles_configs()
    error_msg = validate_with_config_file(post, tiles_configs)
    return error_msg


def validate_with_json(jsoned_data):
    error_msg = None
    try:
        json.loads(jsoned_data)
    except ValueError:
        if LOG:
            print(f'{getTimeStr()} (-) invalid json data: {repr(jsoned_data)}', flush=True)
        error_msg = 'Invalid JSON data.\n'
    return error_msg


def getRedisPrefix(tile_id="*"):
    return f'{PROJECT_NAME}:tile:{tile_id}'


def getTimeStr():
    return datetime.now().strftime("%Hh%M")


def getIsoTime():
    localtime = datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
    tz = '{0:+06.2f}'.format(-float(time.altzone) / 3600)
    iso_time = localtime + tz.replace('.', ':')  # ISO-8601
    return iso_time


def verify_statics(static_file):
    user_tiles_path = os.path.join(
        '.tipboard', '.tipboard/custom_tiles'
    )
    tipboard_tiles_path = os.path.join(TIPBOARD_PATH, 'tiles')
    found = False
    for path in user_tiles_path, tipboard_tiles_path:
        if os.path.exists(os.path.join(path, static_file)):
            found = True
            break
    return found


def tile_path(tile_name):
    """
    Searches for tile's html file (in user's 'custom_tiles' folder,
    and then in app's 'tiles' folder) and returns full path of
    the tile, or raises exception if html file is not present in none
    of those locations.
    """
    user_tiles_path = os.path.join('.tipboard', '.tipboard/custom_tiles')
    tipboard_tiles_path = os.path.join(TIPBOARD_PATH, 'tiles')
    tile_html = '.'.join((tile_name, 'html'))
    for path in user_tiles_path, tipboard_tiles_path:
        tile_path = os.path.join(path, tile_html)
        if os.path.exists(tile_path):
            return tile_path
    raise UserWarning(f'No such tile: {tile_name}')


#!/usr/bin/env python
# -*- coding: utf-8 -*-

from fabric.api import run

from fabtools import require
from fabtools.python import virtualenv


_TIPBOARD_USER = 'tipboard'
_SOURCE_CODE_URL = 'https://github.com/allegro/tipboard.git'


def install():
    configure_os()

    install_tipboard()

    run('chown -R %(user)s:%(user)s /home/%(user)s' % {'user': _TIPBOARD_USER})


def configure_os():
    require.deb.packages([
        'python',
        'python-dev',
        'python-virtualenv',
        'redis-server',
        'libmysqlclient-dev',
        'supervisor',
        'git',
    ])

    require.user(_TIPBOARD_USER, home='/home/' + _TIPBOARD_USER,
                 shell='/bin/bash')


def install_tipboard():
    venv_path = '/home/%s/tipboard-current' % _TIPBOARD_USER

    require.python.virtualenv(venv_path)

    with virtualenv(venv_path):
        require.python.package('distribute==0.6.28')
        require.python.package('tipboard',
                               url=_SOURCE_CODE_URL + '@master#egg=tipboard')

    require.supervisor.process('tipboard',
                               command='%s/bin/tipboard runserver' % venv_path,
                               directory='/home/%s' % _TIPBOARD_USER,
                               user=_TIPBOARD_USER,
                               stdout_logfile='/home/%s/out.log' %
                                              _TIPBOARD_USER)



#!/usr/bin/env python

"""
Jira-ds - a command line utility for fetching various data from Jira.

Usage:
  jira-ds.py --project=NAME (--summary|--cvsr|--issues-counts)
  jira-ds.py --board=NAME (--control-chart|--velocity)
  jira-ds.py --jql=QUERY [-m <limit>]
  jira-ds.py -h | --help

Options:
  --project=NAME   Project name (case sensitive).
  --board=NAME     Board name (case sensitive).
  --jql=QUERY      Return raw data using JQL (Jira Query Language) query.
                   Beware that some queries may be quite heavy on resources!
  -m <limit>       Set 'maxResults' for '--jql' option [default: 1000].
                   (hint: setting it to '0' gives you total number of results
                   without fetching the data)
  --summary        Get issues counts from 'summary' tab.
  --cvsr           Get data table from 'created vs resolved' report
                   (past month).
  --issues-counts  Get issues counts grouped by priority, status, type etc.
  --control-chart  Get data table from 'control chart' report (past month).
  --velocity       Get data table from 'velocity chart' report.
  -h --help        Show this screen.

"""

import json
import os
import requests
import subprocess

from docopt import docopt

JIRA_BASE_URL = ''  # needs to be filled in
JIRA_API_URL = JIRA_BASE_URL + '/rest/api/2/'
GREENHOPPER_API_URL = JIRA_BASE_URL + '/rest/greenhopper/1.0/'
JIRA_CREDENTIALS = {  # needs to be filled in
    'user': '',
    'password': '',
}
SUBP_ENV = os.environ.copy()
SUBP_ENV['JIRA_USER'] = JIRA_CREDENTIALS.get('user')
SUBP_ENV['JIRA_PASSWORD'] = JIRA_CREDENTIALS.get('password')
SUBP_PARAMS = ["casperjs", "jira-ds.js"]
REQUESTS_TIMEOUT = 10  # in seconds


##### 'Internal' stuff.

def _get_identifiers(name, url):
    """
    Gets project_id and project_key (when 'url' points to project) or board_id
    (when 'url' points to board).
    """
    session = requests.Session()
    resp = session.get(url, timeout=REQUESTS_TIMEOUT, params={
        'os_authType': 'basic',
        'os_username': JIRA_CREDENTIALS.get('user'),
        'os_password': JIRA_CREDENTIALS.get('password'),
    })
    resp.raise_for_status()
    rest_data = resp.json()
    try:
        data = rest_data.get('views')  # for boards
    except AttributeError:
        data = rest_data  # for projects
    for d in data:
        if d.get('name') == name:
            return (d.get('id'), d.get('key'))
    return (None, None)  # board/project doesn't exist


def _get_data_from_subprocess(subp_params):
    try:
        data = subprocess.check_output(subp_params, env=SUBP_ENV)
    except subprocess.CalledProcessError as e:
        print("Error in jira-ds.js script: '{}'.".format(e.output.strip()))
        return
    else:
        return json.loads(data)


##### Command-line options.

def get_summary(project_key):
    subp_params = SUBP_PARAMS + ["--project-key=" + project_key,
                                 "--step=summary"]
    return _get_data_from_subprocess(subp_params)


def get_cvsr(project_id):
    subp_params = SUBP_PARAMS + ["--project-id=" + project_id, "--step=cvsr"]
    return _get_data_from_subprocess(subp_params)


def get_issues_counts(project_key):
    subp_params = SUBP_PARAMS + ["--project-key=" + project_key,
                                 "--step=issues-counts"]
    return _get_data_from_subprocess(subp_params)


def get_control_chart(board_id):
    subp_params = SUBP_PARAMS + ["--board-id=" + str(board_id),
                                 "--step=control-chart"]
    return _get_data_from_subprocess(subp_params)


def get_velocity(board_id):
    subp_params = SUBP_PARAMS + ["--board-id=" + str(board_id),
                                 "--step=velocity"]
    return _get_data_from_subprocess(subp_params)


def get_jql(query, max_results):
    session = requests.Session()
    query_url = JIRA_API_URL + 'search?'
    resp = session.get(query_url, timeout=REQUESTS_TIMEOUT, params={
        'os_authType': 'basic',
        'os_username': JIRA_CREDENTIALS.get('user'),
        'os_password': JIRA_CREDENTIALS.get('password'),
        'start-index': 0,
        'maxResults': max_results,
        'jql': query,
    })
    resp.raise_for_status()
    rest_data = resp.json()
    return(rest_data)


##### Putting it all together.

def main():
    args = docopt(__doc__)
    project_name = args.get('--project')
    board_name = args.get('--board')
    jql = args.get('--jql')
    if project_name:
        url = JIRA_API_URL + 'project'
        try:
            project_id, project_key = _get_identifiers(project_name, url)
        except requests.exceptions.RequestException as e:
            print("Your query has raised an error: '{}'.".format(e.message))
            return
        else:
            if not project_id:
                print("Project '{}' does not exist.").format(project_name)
                return
            if args.get('--summary'):
                result = get_summary(project_key)
            elif args.get('--cvsr'):
                result = get_cvsr(project_id)
            elif args.get('--issues-counts'):
                result = get_issues_counts(project_key)
    elif board_name:
        url = GREENHOPPER_API_URL + 'rapidview'
        try:
            board_id, _ = _get_identifiers(board_name, url)
        except requests.exceptions.RequestException as e:
            print("Your query has raised an error: '{}'.".format(e.message))
            return
        else:
            if not board_id:
                print("Board '{}' does not exist.").format(board_name)
                return
            if args.get('--control-chart'):
                result = get_control_chart(board_id)
            elif args.get('--velocity'):
                result = get_velocity(board_id)
    elif jql:
        max_results = args.get('-m')
        try:
            result = get_jql(jql, max_results)
        except requests.exceptions.RequestException as e:
            print("Your query has raised an error: '{}'.".format(e.message))
            return
    print(json.dumps(result, ensure_ascii=False))

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


VERSION = (1, 4, 3)
__version__ = '.'.join(str(num) for num in VERSION)



# -*- coding: utf-8 -*-

# This is an example 'client_code' script which demonstrates how to 'glue'
# together a tile (or collection of tiles) with an external script fetching
# json'ed data from data source.
#
# Script like this one is meant to be launched periodically from crontab.
#
# The main idea here is as follows: you're querying the data source(s) you want,
# transform resulting data according to your needs, and finally you're pushing
# it to your tile, along with optional config data. Really simple, isn't it?

import json
import requests

from subprocess import check_output

# Get your API_KEY from your settings file ('~/.tipboard/settings-local.py').
API_KEY = ''
# Change '127.0.0.1:7272' to the address of your Tipboard instance.
API_URL = 'http://127.0.0.1:7272/api/v0.1/{}'.format(API_KEY)
API_URL_PUSH = '/'.join((API_URL, 'push'))
API_URL_TILECONFIG = '/'.join((API_URL, 'tileconfig'))


#### Helper functions for data transformations (data sources --> tiles).

# We have only one tile here, hence 'prepare_for_pie_chart' is all we need,
# but in real-life scenario you may want some additional functions like
# 'prepare_for_line_chart', 'prepare_for_velocity_chart' and so on.

def prepare_for_pie_chart(data):
    # Pie chart needs data as a list of lists (whose elements are pairs
    # component-percentage), so we have to prepare it.
    data_prepared = []
    for k, v in data.items():
        data_prepared.append([k, v[0]])
    data_prepared = {'title': 'my title', 'pie_data': data_prepared}
    return data_prepared


#### Putting it all together.

def main():
    # Tile 'pie001' (pie chart)
    # (let's say we want to show issues count for project 'Tipboard' grouped by
    # issue status i.e. 'Resolved', 'In Progress', 'Open', 'Closed' etc.)
    TILE_NAME = 'pie_chart'
    TILE_KEY = 'pie001'
    # Now we are launching our script asking data source for the stuff we want:
    # (see its documentation for available options/switches)
    ds_output = check_output(["python", "jira-ds.py", "--project=Tipboard",
                              "--issues-counts"])
    data = json.loads(ds_output)
    data_selected = data.get('status_summary')
    data_prepared = prepare_for_pie_chart(data_selected)
    data_jsoned = json.dumps(data_prepared)
    data_to_push = {
        'tile': TILE_NAME,
        'key': TILE_KEY,
        'data': data_jsoned,
    }
    resp = requests.post(API_URL_PUSH, data=data_to_push)
    if resp.status_code != 200:
        print(resp.text)
        return

    # (optional) config for our pie chart, which is pushed in a separate request
    tile_config = {
        'legend': {
            'show': True,
            'location': 's',
        }
    }
    tile_config_jsoned = json.dumps(tile_config)
    data_to_push = {
        'value': tile_config_jsoned,
    }
    resp = requests.post(
        '/'.join((API_URL_TILECONFIG, TILE_KEY)),
        data=data_to_push,
    )
    if resp.status_code != 200:
        print(resp.text)
        return


    # Tile 'pie002'
    # TILE_NAME = 'pie_chart'
    # TILE_KEY = 'pie002'
    # output = check_output(...
    # ...and the rest in the same manner as in above example, i.e.:
    #  - query the data source for data
    #  - select what you want from that data
    #  - transform selected data according to tile's specs (if needed)
    #  - push transformed data to API_URL_PUSH
    #  - push config data for your tile to API_URL_CONFIG (let's say you want
    #    to override default behavior and show pie chart's legend at
    #    the bottom of the tile, or change some colors etc.)
    # That's all!


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.http import JsonResponse
from django.shortcuts import render

from src.tipboard.app.cache import getCache
from src.tipboard.app.flipboard import Flipboard
from src.tipboard.app.parser import process_layout_config
from src.tipboard.app.properties import *
from src.tipboard.app.utils import tile_path, getTimeStr

cache = getCache()


def flipboardHandler(request):
    if LOG:
        print(f"{getTimeStr()} (+) flipboardHandler/", flush=True)
    data = {
        "page_title": Flipboard().get_flipboard_title(),
        "tipboard_css": TIPBOARD_CSS_STYLES,
        "tipboard_js": ['js/lib/jquery.js', 'js/flipboard.js', 'js/lib/require.js'],
        "flipboard_interval": FLIPBOARD_INTERVAL,
    }
#    data['tipboard_js'].remove('js/tipboard.js')
    return render(request, 'flipboard.html', data)


def getDashboardsPaths(request):
    if LOG:
        print(f"{getTimeStr()} GET /getDashboardsPaths", flush=True)
    paths = Flipboard().get_paths()
    return JsonResponse({'paths': paths}, safe=False)


def dashboardRendererHandler(request, layout_name='layout_config'):
    if LOG:
        print(f"{getTimeStr()} GET dashboardRendererHandler /([a-zA-Z0-9_-]*)", flush=True)
    try:
        config = process_layout_config(layout_name)
        tiles_js = ["tiles/" + '.'.join((name, 'js')) for name in config['tiles_names']]
        #tiles_js = filter(_verify_statics, tiles_js) #TODO fix the verify_statics in utils.py
        tiles_css = ["tiles/" + '.'.join((name, 'css')) for name in config['tiles_names']]
        #tiles_css = filter(_verify_statics, tiles_css) #TODO fix the verify_statics in utils.py
        data = {
            "details": config['details'],
            "layout": config['layout'],
            "tipboard_css": TIPBOARD_CSS_STYLES,
            "tipboard_js": TIPBOARD_JAVASCRIPTS,
            "tiles_css": tiles_css,
            "tiles_js": tiles_js,
            "tile_path": tile_path
        }
    except FileNotFoundError as e:
        if LOG:
            print(f"{getTimeStr()}: (+)Config file:{layout_name} not found", flush=True)
        msg = '<br>'.join([
            '<div style="color: red">',
            f'No config file found for dashboard: {layout_name}',
            f'Make sure that file: "{e.filename}" exists.',
            '</div>',
        ])
        return render(request, 'layout.html', msg)
    return render(request, 'layout.html', data)





from channels.auth import AuthMiddlewareStack
from channels.routing import ProtocolTypeRouter, URLRouter

import src.tipboard.app.routing

application = ProtocolTypeRouter({
    'websocket': AuthMiddlewareStack(
        URLRouter(
            src.tipboard.app.routing.websocket_urlpatterns
        )
    )
})




# -*- coding: utf-8 -*-
import json
from django.http import JsonResponse, HttpResponseBadRequest, HttpResponse, HttpResponseServerError, Http404
from src.tipboard.app.properties import PROJECT_NAME, LAYOUT_CONFIG, REDIS_DB, LOG
from src.tipboard.app.cache import getCache
from src.tipboard.app.utils import getRedisPrefix, getTimeStr, getIsoTime

cache = getCache()
redis = cache.redis


# TODO: "it's better to ask forgiveness than permission" ;)
def getTile(request, tile_key):
    """Handles reading and deleting of tile's data."""
    if redis.exists(tile_key):
        return HttpResponse(redis.get(tile_key))
    else:
        return HttpResponseBadRequest(f"{tile_key} key does not exist.")


def deleteTile(request, tile_key):
    if redis.exists(tile_key):
        redis.delete(tile_key)
        return HttpResponse("Tile's data deleted.")
    else:
        return HttpResponseBadRequest(f"{tile_key} key does not exist.")


def tile(request, tile_key):
    if request.method == "GET":
        return getTile(request, getRedisPrefix(tile_key))
    elif request.method == "DELETE":
        return deleteTile(request, getRedisPrefix(tile_key))
    raise Http404


def createTile(tile_id, value, tile_template):
    dumped_value = json.dumps(dict(
        id=tile_id,
        tile_template=tile_template,
        data=json.loads(value),
        meta={},
        modified=getIsoTime(),
    ))
    cache.set(getRedisPrefix(tile_id), dumped_value)
    return HttpResponse(f"{tile_id} data created successfully.")


def push(request):
    if request.method == "POST":
        try:
            tile_id = request.POST.get("key", None)
            data = request.POST.get("data", None)
            tile_template = request.POST.get("tile", None)
            if not tile_id or not data or not tile_template:
                return HttpResponseBadRequest(f"Missing data")
            tilePrefix = getRedisPrefix(tile_id)
            if not redis.exists(tilePrefix):
                if LOG:
                    print(f"{getTimeStr()}: (+) {tile_id} not found in cache, creating tile {tile_template}", flush=True)
                return createTile(tile_id=tile_id, value=data, tile_template=tile_template)
            cachedTile = json.loads(redis.get(tilePrefix))
            cachedTile['data'] = json.loads(data)
            cachedTile['modified'] = getIsoTime()
            cachedTile['tile_template'] = tile_template
            cache.set(tilePrefix, json.dumps(cachedTile))
            return HttpResponse(f"{tile_id} data updated successfully.")
        except IOError as e:
            return HttpResponseServerError(e)
    raise Http404


def meta(request, tile_key):
    if request.method == "POST":
        try:
            value = request.POST.get("value", None)
            if value is None:
                return HttpResponseBadRequest("Missing data")
            try:
                value = json.loads(value)
            except:
                return HttpResponseBadRequest("Invalid Json data")
            if "big_value_color" not in value or "fading_background" not in value:
                return HttpResponseBadRequest("Bad data")
            tilePrefix = getRedisPrefix(tile_key)
            if not redis.exists(tilePrefix):
                if LOG:
                    print(f"{getTimeStr()}: (+) {tile_key} is not present in cache", flush=True)
                return HttpResponseBadRequest(f"{tile_key} is not present in cache")
            cachedTile = json.loads(redis.get(tilePrefix))
            cachedTile['meta']['big_value_color'] = value['big_value_color']
            cachedTile['meta']['fading_background'] = value['fading_background']
            cache.set(tilePrefix, json.dumps(cachedTile))
            return HttpResponse(f"{tile_key} data updated successfully.")
        except Exception as e:
            return HttpResponseServerError(e)
    raise Http404


# TODO: "it's better to ask forgiveness than permission" ;)
def update(request):
    if request.method == "POST":
        try:
            tile_id = request.POST.get("key", None)
            data = request.POST.get("data", None)
            httpResponse = push(request)
            if httpResponse.status_code != 200:
                return httpResponse
            try:
                request.POST.get("value", None)
                httpResponse = meta(request, tile_id)
                if httpResponse.status_code != 200:
                    return httpResponse
            except Exception as e:
                if LOG:
                    print(f"{getTimeStr()} (-) No meta value for update tile {tile_id}: {e}", flush=True)
                return HttpResponseBadRequest(f"{tile_id} data updated successfully.")
            return HttpResponse(f"{tile_id} data updated successfully.")
        except Exception as e:
            if LOG:
                print(f"{getTimeStr()} (-) Update error: {e}", flush=True)
            return push(request)

def projectInfo(request):
    if request.method == "GET":
        response = {
            'tipboard_version': "v0.1",
            'project_name': PROJECT_NAME,
            'project_layout_config': LAYOUT_CONFIG,
            'redis_db': REDIS_DB,
        }
        return JsonResponse(response)
    raise Http404



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import glob, os, yaml

from src.tipboard.app import properties


class WrongSumOfRows(Exception):
    pass


def _get_tiles_dict(col):
    return list(col.values())[0]


def get_cols(rows):
    #TODO: validation col_1_of_4
    cols = []
    for col in list(rows.values())[0]:
        cols.append(col)
    return cols


def get_rows(layout):
    """Validates and returns number of rows."""
    rows_data = []
    rows_class = []
    for row in layout:
        rows_data.append(row)
        rows_class.append(list(row.keys()))
    rows_count = 0
    sum_of_rows = []
    for row_class in rows_class:
        splited_class = row_class[0].split('_')  # ex: row_1_of_2
        row = splited_class[1]
        of_rows = int(splited_class[3])
        if rows_count == 0:
            rows_count = int(of_rows)
            sum_of_rows.append(int(row))
        elif not rows_count == of_rows:
            raise WrongSumOfRows('The sum of the lines is incorrect.')
        else:
            sum_of_rows.append(int(row))
    if not sum(sum_of_rows) == rows_count:
        raise WrongSumOfRows('The sum of the lines is incorrect.')
    return rows_data


def find_tiles_names(layout):
    name_list, key_list = [], []
    for row in get_rows(layout):
        for col in get_cols(row):
            for tile_dict in _get_tiles_dict(col):
                name = tile_dict['tile_template']
                key = tile_dict['tile_id']
                if key not in key_list:
                    key_list.append(key)
                    if name not in name_list:
                        name_list.append(name)
    return name_list, key_list


def get_config_files_names():
    """
    Return all configs files' names (without '.yaml' ext.) from user space
    (.tipboard/)
    """
    configs_names = []
    configs_dir = os.path.join(properties.user_config_dir, '*.yaml')
    for config_path in glob.glob(configs_dir):
        filename = os.path.basename(config_path)
        head, ext = os.path.splitext(filename)
        configs_names.append(head)
    return configs_names


def config_file_name2path(config_name):
    """
    Return file path to *config_name* (eg. 'layout_config')
    """
    path = os.path.join(
        properties.user_config_dir, ''.join([config_name])
    )
    return path


def get_tiles_configs():
    """
    Return dict with both tiles' keys and ids from all available configs
    """
    tiles_configs = {
        'tiles_keys': set(),
        'tiles_names': set()
    }
    configs_names = get_config_files_names()
    for config_name in configs_names:
        parsed_config = process_layout_config(config_name)
        tiles_configs['tiles_keys'].update(set(parsed_config['tiles_keys']))
        tiles_configs['tiles_names'].update(set(parsed_config['tiles_names']))
    return tiles_configs


def process_layout_config(layout_name='layout_config'):
    config_path = config_file_name2path(layout_name)
    try:
        with open(config_path, 'r') as layout_config:
            config = yaml.load(layout_config)
    except FileNotFoundError:
        if ".yaml" not in config_path:
            config_path += ".yaml"
        with open(config_path, 'r') as layout_config:
            config = yaml.load(layout_config)
    layout = config['layout']
    config['tiles_names'], config['tiles_keys'] = find_tiles_names(layout)
    return config



import json, os

dir_path = os.path.dirname(os.path.realpath(__file__))
conf = json.load(open(dir_path + '/Config/properties.json'))

FORMAT_SIMPLE_DATE = '%Y-%m-%d'
API_VERSION = conf['API_VERSION']
API_KEY = conf['TIPBOARD_TOKEN']
PROJECT_NAME = conf['PROJECT_NAME']
SUPER_SECRET_KEY = conf["SUPER_SECRET_KEY"]
debug = conf['DEBUG']
LOG = conf['LOG']
VERSION = conf["VERSION"]
SITE_ENV = conf["SITE_ENV"]
LOCAL = conf['LOCAL']
CDN_URL = conf['CDN_URL'] #if you are in production and need a CDN for media and static file

REDIS_HOST = conf['REDIS_HOST']
REDIS_PORT = conf['REDIS_PORT']
REDIS_PASSWORD = conf['REDIS_PASSWORD']
REDIS_DB = conf['REDIS_DB']

# Location of Tipboard sources
TIPBOARD_PATH = os.path.dirname(__file__)

# Javascript log level ('1' for 'standard', '2' for 'debug')
JS_LOG_LEVEL = 2

# Our default color palette
COLORS = {
    'black':            '#000000',
    'white':            '#FFFFFF',
    'tile_background':  '#15282d',
    'red':              '#d50000',
    'yellow':           '#ffea00',
    'green':            '#00c853',
    'blue':             '#0091ea',
    'violet':           '#aa00ff',
    'orange':           '#ff6d00',
    'naval':            '#00bfa5',
}

# how many seconds dashboard is displayed before is flipped
FLIPBOARD_INTERVAL = 0
# file name(s) of EXISTING layouts without extension, eg. ['layout_config']
FLIPBOARD_SEQUENCE = []


TIPBOARD_CSS_STYLES = [
    'css/reset.css',
    'css/jquery.jqplot.css',
    'css/layout.css',
]
TIPBOARD_JAVASCRIPTS = [
    'js/lib/jquery.js',
    'js/lib/simplify.js',
    'js/lib/jquery.fullscreen.js',
    'js/lib/jqplot/jquery.jqplot.js',
    'js/lib/jqplot/plugins/jqplot.trendline.js',
    'js/lib/jqplot/plugins/jqplot.canvasAxisTickRenderer.js',
    'js/lib/jqplot/plugins/jqplot.canvasTextRenderer.js',
    'js/lib/jqplot/plugins/jqplot.categoryAxisRenderer.js',
    'js/lib/jqplot/plugins/jqplot.barRenderer.js',
    'js/lib/jqplot/plugins/jqplot.pointLabels.js',
    'js/lib/jqplot/plugins/jqplot.highlighter.js',
    'js/lib/jqplot/plugins/jqplot.dateAxisRenderer.js',
    'js/lib/jqplot/plugins/jqplot.pieRenderer.js',
    'js/lib/jqplot/plugins/jqplot.blockRenderer.js',
    'js/lib/jqplot/plugins/jqplot.bubbleRenderer.js',
    'js/lib/jqplot/plugins/jqplot.canvasAxisLabelRenderer.js',
    'js/lib/jqplot/plugins/jqplot.canvasOverlay.js',
    'js/lib/jqplot/plugins/jqplot.ciParser.js',
    'js/lib/jqplot/plugins/jqplot.cursor.js',
    'js/lib/jqplot/plugins/jqplot.donutRenderer.js',
    'js/lib/jqplot/plugins/jqplot.dragable.js',
    'js/lib/jqplot/plugins/jqplot.enhancedLegendRenderer.js',
    'js/lib/jqplot/plugins/jqplot.funnelRenderer.js',
    'js/lib/jqplot/plugins/jqplot.json2.js',
    'js/lib/jqplot/plugins/jqplot.logAxisRenderer.js',
    'js/lib/jqplot/plugins/jqplot.mekkoAxisRenderer.js',
    'js/lib/jqplot/plugins/jqplot.mekkoRenderer.js',
    'js/lib/jqplot/plugins/jqplot.meterGaugeRenderer.js',
    'js/lib/jqplot/plugins/jqplot.mobile.js',
    'js/lib/jqplot/plugins/jqplot.ohlcRenderer.js',
    'js/lib/jqplot/plugins/jqplot.pyramidAxisRenderer.js',
    'js/lib/jqplot/plugins/jqplot.pyramidGridRenderer.js',
    'js/lib/jqplot/plugins/jqplot.pyramidRenderer.js',
    'js/tipboard.js',
]

FROM_PIP = 'src.'
#FROM_PIP = ''

#Tu dois faire la diff des path quand
# * dans pip
# * dans docker
# * dans bash

# Determine which layout config should be used
user_config_dir = dir_path + '/Config/'

LAYOUT_CONFIG = os.path.join(user_config_dir, 'layout_config.yaml')




# -*- coding: utf-8 -*-
import json, redis
from asgiref.sync import async_to_sync
from src.tipboard.app.properties import REDIS_DB, REDIS_PASSWORD, REDIS_HOST, REDIS_PORT, LOCAL, LOG
from src.tipboard.app.utils import getTimeStr, getRedisPrefix

cache = None


def getCache():
    global cache
    if cache is None:
        cache = MyCache()
    return cache


class MyCache:
    def __init__(self):
        try:
            self.redis = self.setup_redis()
            self.redis.time()
            self.isRedisConnected = True
            if LOG:
                print(f"{getTimeStr()} (+) Initializing cache from redis server with {len(self.listOfTilesCached())} key", flush=True)
            self.clientsWS = list()
        except:
            self.isRedisConnected = False
        pass

    def delete(self, tile_id, value=None, tile=None):
        if self.redis.exists(getRedisPrefix(tile_id=tile_id)):
            self.redis.delete(getRedisPrefix(tile_id=tile_id))
            return True
        if LOG:
            print(f"{getTimeStr()}(-) tile: {tile_id} not found in redis", flush=True)
        return False

    def get(self, tile_id):
        if LOG:
            print(f"cache.get({tile_id})", flush=True)
        prefix = tile_id
        if self.redis.exists(prefix):
            return json.dumps(self.redis.get(prefix))
        if LOG:
            print(f"{getTimeStr()} (-) tile: {prefix} not found in redis", flush=True)
        return None

    def setup_redis(self):
        if LOCAL:
            return redis.StrictRedis(host="0.0.0.0", port=REDIS_PORT, password=None, decode_responses=True, db=REDIS_DB)
        else:
            return redis.StrictRedis(host=REDIS_HOST, port=REDIS_PORT, password=REDIS_PASSWORD, decode_responses=True,
                                     db=REDIS_DB)

    def set(self, key, dumped_value):
        self.redis.set(key, dumped_value)
        if LOG:
            print(f"{getTimeStr()} (+) Redis save and publish: {key}", flush=True)
        key = key.split(":")[-1]
        from channels.layers import get_channel_layer
        channel_layer = get_channel_layer()
        async_to_sync(channel_layer.group_send)("event", {"type": "update.tile",  "tile_id": key}, )

    def listOfTilesCached(self):
        listOfTiles = list()
        for key in self.redis.keys(getRedisPrefix()):
            listOfTiles.append(key)
        return listOfTiles




"""
ASGI entrypoint. Configures Django and then runs the application
defined in the ASGI_APPLICATION setting.
"""

import os
import django
from channels.routing import get_default_application

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'webserver.settings')
django.setup()
application = get_default_application()


# -*- coding: utf-8 -*-
from src.tipboard.app.utils import getTimeStr
from src.tipboard.app.properties import *
from src.tipboard.app.parser import get_config_files_names, process_layout_config

class Flipboard(object):

    def __init__(self):
        self.last_found_configs_number = -1
        self.paths = []

    def get_paths(self):
        """
        Returns url paths to dashboards (created from .yaml config from
        userspace).
        """
        config_names = self._get_config_names()
        if len(config_names) != self.last_found_configs_number:
            self.paths = self._config_names2paths(config_names)
            self.last_found_configs_number = len(config_names)
        return self.paths

    def get_flipboard_title(self):
        """
        Returns title to display as a html title.
        """
        title = ''
        config_names = self._get_config_names()
        if len(config_names) == 1:
            config = process_layout_config(config_names[0])
            try:
                title = config['details']['page_title']
            except KeyError:
                msg = 'config {} has no key: details/page_title'.format(
                    config_names[0]
                )
                if LOG:
                    print(f"{getTimeStr()} (+) {msg}", flush=True)
        elif len(config_names) > 1:
            # TODO: put here more suitable title?
            title = 'Flipboard Mode'
        return title

    def _get_config_names(self):
        config_names = FLIPBOARD_SEQUENCE
#        print("settings.FLIPBOARD_SEQUENCE: " + str(FLIPBOARD_SEQUENCE))
        if not any(config_names):
            config_names = get_config_files_names()
            if not any(config_names):

                raise Exception('No config (.yaml) file found in ./tipboard/app/Config/layout_config.yaml')
        return config_names

    def _config_names2paths(self, config_names):
        paths = []
        for name in config_names:
            path = '/' + name
            paths.append(path)
        return paths


import pathlib, sys
from setuptools import setup, find_packages

assert sys.version_info >= (3, 7), "Python 3.7+ required."

# The directory containing this file
HERE = pathlib.Path.cwd()


# The text of the README file
README = (HERE / "README.md").read_text()

print(README)

with open(HERE / 'requirements.txt') as requirements:
    required = requirements.read().splitlines()

from src import __version__

setup(
    name="tipboard2.0",
    version=__version__,
    description="Tipboard - a flexible solution for creating your dashboards.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="http://tipboard.allegrogroup.com",
    author="Allegro Group and Contributors",
    author_email="pylabs@allegro.pl",
    license='Apache Software License v2.0',
    packages=find_packages(),
    include_package_data=True,
    install_requires=required,
    entry_points={
        'console_scripts': [
            'tipboard = src.manage:main_as_pkg',
        ],
    },
)





# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from src.tipboard.app.cache import getCache
from src.tipboard.app.properties import COLORS, JS_LOG_LEVEL, LOG
from src.tipboard.app.utils import getRedisPrefix, getTimeStr
from channels.generic.websocket import WebsocketConsumer
from asgiref.sync import async_to_sync

cache = getCache()
tipboard_helpers = {
    'color': COLORS,
    'log_level': JS_LOG_LEVEL,
}


class ChatConsumer(WebsocketConsumer):
    """Handles client connections on web sockets and listens on a Redis
    subscription."""

    def connect(self):
        #self.channel_name = "events"
        async_to_sync(self.channel_layer.group_add)("event", self.channel_name)
        if LOG:
            print(f"{getTimeStr()} (+) WS: New client with channel:{self.channel_name}", flush=True)
        self.accept()
        for tile_id in cache.listOfTilesCached():
            self.update_tile_receive(tile_id=tile_id)

    def disconnect(self, close_code):
        if LOG:
            print(f"{getTimeStr()} (+) WS: client with channel:{self.channel_name} disconnected", flush=True)
        async_to_sync(self.channel_layer.group_discard)("event", self.channel_name)

    def receive(self, text_data, **kwargs):
        #If msg is receive, client wants the full list of tiles
        if LOG:
            print(f"{getTimeStr()} (+) WS: sending tiles to :{self.channel_name}", flush=True)
        for tile_id in cache.listOfTilesCached():
            self.update_tile_receive(tile_id=tile_id)

    def update_tile_receive(self, tile_id):
        tileData = cache.get(tile_id=tile_id)
        if tileData is None:
            if LOG:
                print(f'{getTimeStr()} (-) No data in key {tile_id} on Redis.', flush=True)
            #                stale_keys.add(tile_id)
            return
        data = json.loads(tileData)
        if type(data) is str:
            data = json.loads(data)
        data['tipboard'] = tipboard_helpers
        self.send(text_data=json.dumps(data))

    def update_tile(self, data):
        tile_id = getRedisPrefix(data['tile_id'])
        tileData = cache.get(tile_id=tile_id)
        if tileData is None:
            if LOG:
                print(f'{getTimeStr()} (-) No data in key {tile_id} on Redis.', flush=True)
            return
        data = json.loads(tileData)
        if type(data) is str:
            data = json.loads(data)
        data['tipboard'] = tipboard_helpers
        self.send(text_data=json.dumps(data))



from django import template

register = template.Library()

ALLOWED_TILES = ["text", "simple_percentage", "pie_chart", "norm_chart", "listing", "line_chart", "just_value",
                 "fancy_listing", "empty", "cumulative_flow", "big_value", "bar_chart", "advanced_plot"]
from django.template.loader import render_to_string

@register.filter(name ="template_tile")
def template_tile(tile_id, tile_data):
    """
    return a String, this is the render of the tile template
    :param tile_id:
    :param tile_data:
    :return:
    """
    if type(tile_data) is dict and tile_data['tile_template'] in ALLOWED_TILES:
        return render_to_string(f"tiles/{tile_data['tile_template']}.html",
                                {'tile_id': tile_id, "title": tile_data['title']})
    return ""